"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const terminal_link_1 = __importDefault(require("terminal-link"));
const picocolors_1 = __importDefault(require("picocolors"));
const node_fetch_1 = __importDefault(require("node-fetch"));
const form_data_1 = __importDefault(require("form-data"));
const is_ci_1 = __importDefault(require("is-ci"));
const sizeCheckBundles_1 = require("./sizeCheckBundles");
const read_pkg_up_1 = __importDefault(require("read-pkg-up"));
const git_1 = require("./git");
const pluginName = 'InItStatsWebpackPlugin';
// const log = (...args: any[]) => console.log(...args.map((a) => pc.bold(pc.gray(a))));
class InItStatsWebpackPlugin {
    constructor(options) {
        this.options = options;
    }
    apply(compiler) {
        compiler.hooks.done.tapPromise(pluginName, (compilation) => __awaiter(this, void 0, void 0, function* () {
            var _a, _b, _c, _d, _e, _f, _g;
            const exists = yield fs_extra_1.default.exists(this.options.reportFilename);
            if (exists) {
                const fileStat = yield fs_extra_1.default.stat(this.options.reportFilename);
                const emptyStats = fileStat.size < 3;
                if (!this.options.serverUrl || emptyStats) {
                    //   console.warn(`in-it stats serverUrl is not defined`);
                    return;
                }
                const serverUrl = new URL(this.options.serverUrl);
                const file = yield fs_extra_1.default.readFile(this.options.reportFilename);
                const appPackage = yield (0, read_pkg_up_1.default)();
                const generatorPackage = yield (0, read_pkg_up_1.default)({ cwd: __dirname });
                const userEmail = yield (0, git_1.getUserEmail)();
                const userName = yield (0, git_1.getUserName)();
                const branch = yield (0, git_1.getCurrentBranch)();
                const remoteUrl = yield (0, git_1.getRemoteUrl)();
                const commitHash = yield (0, git_1.getCommitHash)();
                const provider = (_a = process.env.VERCEL_GIT_PROVIDER) !== null && _a !== void 0 ? _a : getProviderFromUrl(remoteUrl);
                const repository = process.env.VERCEL_GIT_REPO_OWNER && process.env.VERCEL_GIT_REPO_SLUG
                    ? `${process.env.VERCEL_GIT_REPO_OWNER}/${process.env.VERCEL_GIT_REPO_SLUG}`
                    : getRepositoryFromUrl(remoteUrl);
                const gitRootDir = yield (0, git_1.getRootDir)();
                const packagePath = gitRootDir ? path_1.default.relative(gitRootDir, process.cwd()) : process.cwd();
                const compilation = path_1.default.basename(this.options.reportFilename, '.json');
                const formData = new form_data_1.default();
                formData.append('file', file, this.options.reportFilename);
                setFormData(formData, 'environment', is_ci_1.default ? 'ci' : 'local');
                setFormData(formData, 'buildId', this.options.buildId);
                setFormData(formData, 'version', appPackage === null || appPackage === void 0 ? void 0 : appPackage.packageJson.version); // from app package.json
                setFormData(formData, 'name', this.options.name);
                setFormData(formData, 'userEmail', userEmail); // from git config
                setFormData(formData, 'userName', userName); // from git config
                setFormData(formData, 'provider', provider); // from git config remote url
                setFormData(formData, 'compilation', compilation); // from webpack stats
                setFormData(formData, 'branch', branch); // from git branch
                setFormData(formData, 'generatingTool', (_b = generatorPackage === null || generatorPackage === void 0 ? void 0 : generatorPackage.packageJson) === null || _b === void 0 ? void 0 : _b.name); // TOOD: maybe different endpoint
                setFormData(formData, 'generatingToolVersion', (_c = generatorPackage === null || generatorPackage === void 0 ? void 0 : generatorPackage.packageJson) === null || _c === void 0 ? void 0 : _c.version); // TOOD: get from package.json
                setFormData(formData, 'repository', repository); // from git config remote url
                setFormData(formData, 'packagePath', packagePath); // from build dir
                setFormData(formData, 'packageName', (_d = appPackage === null || appPackage === void 0 ? void 0 : appPackage.packageJson) === null || _d === void 0 ? void 0 : _d.name); // from app package.json
                setFormData(formData, 'commitHash', commitHash); // from git branch
                if (process.env.DEBUG) {
                    console.log({
                        environment: process.env.NODE_ENV === 'production' && is_ci_1.default ? 'ci' : 'local',
                        buildId: this.options.buildId,
                        version: appPackage === null || appPackage === void 0 ? void 0 : appPackage.packageJson.version,
                        name: this.options.name,
                        userEmail,
                        userName,
                        provider,
                        compilation,
                        branch,
                        generatingTool: (_e = generatorPackage === null || generatorPackage === void 0 ? void 0 : generatorPackage.packageJson) === null || _e === void 0 ? void 0 : _e.name,
                        generatingToolVersion: (_f = generatorPackage === null || generatorPackage === void 0 ? void 0 : generatorPackage.packageJson) === null || _f === void 0 ? void 0 : _f.version,
                        repository,
                        packagePath,
                        packageName: (_g = appPackage === null || appPackage === void 0 ? void 0 : appPackage.packageJson) === null || _g === void 0 ? void 0 : _g.name,
                        commitHash,
                    });
                    return;
                }
                const response = yield (0, node_fetch_1.default)(serverUrl.toString(), {
                    method: 'POST',
                    body: formData,
                });
                if (!response.ok) {
                    // console.log(response);
                    console.log(picocolors_1.default.red(response.statusText));
                    console.log(picocolors_1.default.red(`Can't send in-it stats to the server`));
                }
                else {
                    const data = (yield response.json());
                    const name = path_1.default.basename(this.options.reportFilename, '.json');
                    console.log(picocolors_1.default.green((0, terminal_link_1.default)(`Analyze ${picocolors_1.default.bold(name)} Bundle`, `${serverUrl.protocol}${serverUrl.host}${data.url}`)));
                }
            }
            else {
                console.log(picocolors_1.default.yellow(`in-it stats "${this.options.reportFilename}" does not exist`));
            }
            yield (0, sizeCheckBundles_1.sizeCheckBundles)({ outDir: this.options.outDir });
        }));
    }
}
exports.default = InItStatsWebpackPlugin;
function getProviderFromUrl(url) {
    if (!url) {
        return;
    }
    const match = url.match(/^(https?:\/\/|git@)([^\\/:]+)[\\/:]/);
    if (match) {
        if (match[2] === 'github.com') {
            return 'github';
        }
        if (match[2] === 'gitlab.com') {
            return 'gitlab';
        }
        if (match[2] === 'bitbucket.org') {
            return 'bitbucket';
        }
        return match[2];
    }
}
function getRepositoryFromUrl(providerUrl) {
    if (!providerUrl) {
        return;
    }
    const match = providerUrl.match(/^(https?:\/\/|git@)([^\\/:]+)[\\/:](.+?)(?:\.git)?$/);
    if (match) {
        return match[3];
    }
}
function setFormData(formData, key, value) {
    if (value) {
        formData.append(key, value);
    }
}
